# ! Callback Test Setup Version 583
import math,random

# Version 583
# I've now got the figure to move along a randomly wiggling path, using a couple of things I spotted in these callback samples:
# 1) The frame number is used to calculate the parameter's value in C:\Program Files\Poser Software\Poser 11\Runtime\Python\poserScripts\SampleCallbacks\parmCallback.py
# 2) Global variables are used in C:\Program Files\Poser Software\Poser 11\Runtime\Python\poserScripts\SampleCallbacks\randomHeadVerts.py a
# The main reason for using globals in randomHeadVerts.py is for speed, i.e. precalculating a list of random numbers. I'll probably want to do that later, but that's not my main reason for using globals.
# I want to update xTran and zTran with a simple mathematical calculation on each frame. 
# However, the callbacks are called multiple times, which means that my calculations are done multiple times, giving an incorrect result.
# By saving the previous value AND the previous frame number as globals I can get around this

# INSTRUCTIONS FOR RUNNING TEST 583
# ---------------------------------
# 1) Load any figure and ensure its body is selected (figures that face along the +ve Z axis will look best)
# 2) [Optional:Increase the frame count from 30 if you want - 30 is enough to prove the principle, but 360 is more fun]
# 3) Run this script
# 4) Use the |> Step Forward button on the Animation Controls pane to go frame by frame - the figure will move along a wiggling path
# N.B. At present this only works as intended if you step forward frame by frame, or slide the frame counter from left to right once. This is because the update check is simply 'if curframe != ###Frame:'

# Callback for the "yRotate" parameter
def cheapskatesYRotateCallback(parm,value):
	global yRot
	global yRotFrame
	curframe = poser.Scene().Frame()
	if curframe != yRotFrame:
		yRot = (yRot + (30 * (random.random()-0.5))) % 360
		yRotFrame = curframe
	return yRot

# Callback for the "xTran" parameter
def cheapskatesXTranCallback(parm,value):
	global xPos
	global xPosFrame
	curframe = poser.Scene().Frame()
	if curframe != xPosFrame:
		xPos = xPos + (onestep*math.sin(math.radians(parmYRot.Value())))
		xPosFrame = curframe
	return xPos
	
# Callback for the "zTran" parameter
def cheapskatesZTranCallback(parm,value):
	global zPos
	global zPosFrame
	curframe = poser.Scene().Frame()
	if curframe != zPosFrame:
		zPos = zPos + (onestep*math.cos(math.radians(parmYRot.Value())))
		zPosFrame = curframe
	return zPos

# Need to recalculate yRot,XTran, and zTran each frame based on calculated ('keyed', upper figure on dial) value from previous frame, NOT the dialed ('natural', lower figure on dial)
# This (i.e. using globals) seemed the easiest way to do it, at least for my first attempt
global yRot
global yRotFrame
global xPos
global xPosFrame
global zPos
global zPosFrame

# These are just to avoid having to do 'body.Parameter("yRotate")' etc in the callbacks. There are others I could add too.
global parmXTran
global parmZTran
global onestep

# Initialize a few things
body=poser.Scene().CurrentFigure().ActorByInternalName("BODY")
parmYRot=body.Parameter("yRotate")
yRot=0
yRotFrame=0
xPos=0
xPosFrame=0
zPos=0
zPosFrame=0
parmXTran=body.Parameter("xTran")
parmZTran=body.Parameter("zTran")
onestep=0.1
	
# Kick off the callbacks
print "Setting up the 'yRotate' callback..."
poser.Scene().CurrentActor().Parameter("yRotate").SetUpdateCallback(cheapskatesYRotateCallback)
print "Setting up the 'xTran' callback..."
poser.Scene().CurrentActor().Parameter("xTran").SetUpdateCallback(cheapskatesXTranCallback)
print "Setting up the 'zTran' callback..."
poser.Scene().CurrentActor().Parameter("zTran").SetUpdateCallback(cheapskatesZTranCallback)
print "...done."
